@@ -46,7 +46,7 @@ pkt_handler_t pkt_handlers[ESTATE_SIZE][PKTTYPE_SIZE];
 
 uint8_t default_session_key[32];
 
-/** Null packet handler 
+/** Null packet handler
  *
  * Handler for packets which have no action in the current state.
  *
@@ -55,13 +55,13 @@ uint8_t default_session_key[32];
  * @param count The length of the data in \a buf.
  * @return The state machine state to change to.
  */
-static ekey_state_t 
+static ekey_state_t
 null_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
     return state->current_state;
 }
 
-/** Bad key packet handler 
+/** Bad key packet handler
  *
  * Handler for packets which indicate a bad key was set up.
  *
@@ -70,13 +70,13 @@ null_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
  * @param count The length of the data in \a buf.
  * @return The unstrusted state machine state.
  */
-static ekey_state_t 
+static ekey_state_t
 badkey_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
     return ESTATE_UNTRUSTED;
 }
 
-/** Reset packet handler 
+/** Reset packet handler
  *
  * Handler for packets which cause the reset of the entropy key connection.
  *
@@ -85,7 +85,7 @@ badkey_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
  * @param count The length of the data in \a buf, should be 0.
  * @return The initial state machine state.
  */
-static ekey_state_t 
+static ekey_state_t
 reset_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
     char reset[] = {0x3};
@@ -93,7 +93,7 @@ reset_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
     epkt_setsessionkey(state->epkt, state->snum, default_session_key);
 
     state->con_reset++; /* Update the connection statistics */
-    
+
 #ifdef SBQS_MESSAGES
     printf("DEBUG Connection reset\n");
 #endif
@@ -101,7 +101,7 @@ reset_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 }
 
 
-/** Information packet handler 
+/** Information packet handler
  *
  * Handler for packets containing information from the entropy device.
  *
@@ -110,7 +110,7 @@ reset_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
  * @param count The length of the data in \a buf.
  * @return The current state machine state as info packets do not change current state.
  */
-static ekey_state_t 
+static ekey_state_t
 info_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
     char *next = (char *)buf;
@@ -197,7 +197,7 @@ info_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
     return state->current_state;
 }
 
-/** Key request packet handler 
+/** Key request packet handler
  *
  * Handler for packets which cause a session keying request. A K packet with a
  * suitable nonce is issued.
@@ -207,7 +207,7 @@ info_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
  * @param count The length of the data in \a buf.
  * @return The session keying state machine state.
  */
-static ekey_state_t 
+static ekey_state_t
 keyreq_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
     char sbuf[32];
@@ -217,17 +217,17 @@ keyreq_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
         state->nonce_len = 12;
         state->nonce = malloc(state->nonce_len);
     }
-    
-    if (!fill_nonce(state->nonce + sizeof(uint32_t), 
+
+    if (!fill_nonce(state->nonce + sizeof(uint32_t),
                     state->nonce_len - sizeof(uint32_t))) {
-        char serialnumber[32];
-        
-        if (econ_getsnum(state, serialnumber))
-            serialnumber[12] = 0;
-        else
-            strcpy(serialnumber, "UnknownKey");
-    
-        syslog(LOG_ERR, "%s: Unable to prepare nonce for keying.  Key no longer trusted.", serialnumber);
+        char *serialnumber = econ_getsnum(state);
+
+        if (serialnumber != NULL) {
+            syslog(LOG_ERR, "%s: Unable to prepare nonce for keying.  Key no longer trusted.", serialnumber);
+            free(serialnumber);
+        } else {
+            syslog(LOG_ERR, "UnknownKey: Unable to prepare nonce for keying.  Key no longer trusted.");
+        }
         return ESTATE_UNTRUSTED;
     }
     *(uint32_t *)state->nonce = state->con_nonces;
@@ -259,33 +259,44 @@ keyreq_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 static ekey_state_t
 keyreq_count_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
-    char serialnumber[32];
-    
-    if (econ_getsnum(state, serialnumber))
-        serialnumber[12] = 0;
-    else
-        strcpy(serialnumber, "UnknownKey");
-    
+    char *serialnumber;
+
+    serialnumber = econ_getsnum(state);
+    if (serialnumber == NULL) {
+        serialnumber = strdup("UnknownKey");
+        if (serialnumber == NULL) {
+            /* Complete faliure to allocate serial number string. Either system
+             * is unable to fufil the small request or heap is corrupt. Only
+             * course of action is to reset the key.
+             */
+            syslog(LOG_ERR, "Unable to allocate serial number string buffer. Resetting state machine and device.");
+
+            return reset_pkt_handler(state, buf, count);
+        }
+    }
+
     if (state->keyreq_counter++ < MAX_PACKETS_BEFORE_RESET) {
         syslog(LOG_WARNING, "%s: Repeated key request (ignored)", serialnumber);
+        free(serialnumber);
         return ESTATE_SESSION_SENT;
     }
-    
+
     syslog(LOG_WARNING, "%s: Too many key requests in a row. Resetting state machine and device.", serialnumber);
-    
+    free(serialnumber);
+
     return reset_pkt_handler(state, buf, count);
 }
 
-/** Session rekeying packet handler 
+/** Session rekeying packet handler
  *
- * Handler for session rekeying response packets 
+ * Handler for session rekeying response packets
  *
  * @param state The current connection state.
  * @param buf A nonce from the key to mix with our state.
  * @param count The length of the nonce in \a buf.
  * @return The keyed session state machine state or resets to connection.
  */
-static ekey_state_t 
+static ekey_state_t
 key_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
     short nonce_len;
@@ -310,17 +321,17 @@ key_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
     PrepareSkein(&state->session_state, state->snum, session_key, EKEY_SKEIN_PERSONALISATION_EES);
 
     state->con_rekeys++;
-    
+
 #ifdef SBQS_MESSAGES
     printf("DEBUG Rekeying completed\n");
     printf("ACTIVITY Consuming a single FIPS state\nSIZE 4095\n");
     sbqs_start_timing();
 #endif
-    
+
     return ESTATE_KEYED_FIRST;
 }
 
-/** Entropy packet handler 
+/** Entropy packet handler
  *
  * Handler for entropy from the device.
  *
@@ -329,7 +340,7 @@ key_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
  * @param count The length of the entropy in \a buf.
  * @return The keyed state machine state.
  */
-static ekey_state_t 
+static ekey_state_t
 entropy_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
 {
     EKeySkein session_state;
@@ -376,7 +387,7 @@ entropy_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
     return ESTATE_KEYED;
 }
 
-/** Serial number packet handler 
+/** Serial number packet handler
  *
  * Handler for serial number packets.
  *
@@ -385,7 +396,7 @@ entropy_pkt_handler(econ_state_t *state, uint8_t *buf, size_t count)
  * @param count The length of the serial number in \a buf.
  * @return The session keying required state machine state.
  */
-static ekey_state_t 
+static ekey_state_t
 snum_pkt_handler(econ_state_t *state, uint8_t *snum, size_t count)
 {
     /* bad serial number length - reset the conenction */
@@ -428,7 +439,7 @@ snum_pkt_handler(econ_state_t *state, uint8_t *snum, size_t count)
     return ESTATE_SESSION;
 }
 
-/** Sets the default handler for connection state machine. 
+/** Sets the default handler for connection state machine.
  *
  * @param state The state being setup.
  */
@@ -463,7 +474,7 @@ init_states(void)
 
     pkt_handlers[ESTATE_SESSION][PKTTYPE_INFO] = info_pkt_handler;
     pkt_handlers[ESTATE_SESSION][PKTTYPE_KEYREQ] = keyreq_pkt_handler;
-    
+
     pkt_handlers[ESTATE_SESSION_SENT][PKTTYPE_KEYREQ] = keyreq_count_pkt_handler;
     pkt_handlers[ESTATE_SESSION_SENT][PKTTYPE_INFO] = info_pkt_handler;
     pkt_handlers[ESTATE_SESSION_SENT][PKTTYPE_KEY] = key_pkt_handler;
@@ -555,29 +566,29 @@ int econ_run(econ_state_t *con_state)
     return res;
 }
 
-/** Get a connections state machine state. 
+/** Get a connections state machine state.
  *
  * @param con_state The connection context.
  * @return The state machine state.
  */
-ekey_state_t 
+ekey_state_t
 econ_state(econ_state_t *con_state)
 {
     return con_state->current_state;
 }
 
-/** Get a connections output file descriptor. 
+/** Get a connections output file descriptor.
  *
  * @param con_state The connection context.
  * @return The file descriptor.
  */
-int 
+int
 econ_get_rd_fd(econ_state_t *con_state)
 {
     return con_state->key_stream->fd;
 }
 
-/** Set the a connections serial number. 
+/** Set the a connections serial number.
  *
  * @param con_state The connection context.
  * @param snum The pem encoded serial number.
@@ -589,23 +600,25 @@ econ_setsnum(econ_state_t *state, const char *snum)
     state->snum_len = pem64_decode_bytes(snum, 16, state->snum);
 }
 
-/** Get the a connections pem encoded serial number. 
- *
- * @param state The connection context.
- * @param snum The serial number to set.
- * @return True if the seral number buffer was updated.
- */
-bool
-econ_getsnum(econ_state_t *state, char *snum)
+/* exported interface, documented in connection.h */
+char *
+econ_getsnum(econ_state_t *state)
 {
-    if (state->snum == NULL)
-        return false;
+    char *serial_str;
 
-    pem64_encode_bytes(state->snum, state->snum_len, snum);
-    return true;
+    if (state->snum == NULL)
+        return NULL;
+
+    serial_str = calloc(1, pem64_encode_bytes_length(state->snum_len) + 1);
+    if (serial_str == NULL)
+        return NULL;
+
+    pem64_encode_bytes(state->snum, state->snum_len, serial_str);
+
+    return serial_str;
 }
 
-/** Shutdown a connection and free its context. 
+/** Shutdown a connection and free its context.
  *
  * @param state The connection context.
  * @return zero.
